@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2003-2004 Imendio AB
- * Copyright (C) 2003      Colin Walters <walters@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -71,7 +70,6 @@ ssl_verify_certificate (LmSSL *ssl, const gchar *server)
 	}
 	
 	if (status & GNUTLS_CERT_INVALID
-	    || status & GNUTLS_CERT_NOT_TRUSTED
 	    || status & GNUTLS_CERT_REVOKED) {
 		if (ssl->func (ssl, LM_SSL_STATUS_UNTRUSTED_CERT,
 			       ssl->func_data) != LM_SSL_RESPONSE_CONTINUE) {
@@ -95,8 +93,8 @@ ssl_verify_certificate (LmSSL *ssl, const gchar *server)
 	
 	if (gnutls_certificate_type_get (ssl->gnutls_session) == GNUTLS_CRT_X509) {
 		const gnutls_datum* cert_list;
-		int cert_list_size;
-		int digest_size;
+		guint cert_list_size;
+		size_t digest_size;
 		gnutls_x509_crt cert;
 		
 		cert_list = gnutls_certificate_get_peers (ssl->gnutls_session, &cert_list_size);
@@ -126,7 +124,7 @@ ssl_verify_certificate (LmSSL *ssl, const gchar *server)
 
 		gnutls_x509_crt_deinit (cert);
 
-		if (gnutls_x509_fingerprint (GNUTLS_DIG_MD5, &cert_list[0],
+		if (gnutls_fingerprint (GNUTLS_DIG_MD5, &cert_list[0],
 					     ssl->fingerprint,
 					     &digest_size) >= 0) {
 			if (ssl->expected_fingerprint &&
@@ -204,15 +202,21 @@ GIOStatus
 _lm_ssl_read (LmSSL *ssl, gchar *buf, gint len, gsize *bytes_read)
 {
 	GIOStatus status;
-	
-	*bytes_read = gnutls_record_recv (ssl->gnutls_session, buf, len);
-	
-	if (*bytes_read == GNUTLS_E_AGAIN) {
+	gint      b_read;
+
+	*bytes_read = 0;
+	b_read = gnutls_record_recv (ssl->gnutls_session, buf, len);
+
+	if (b_read == GNUTLS_E_AGAIN) {
 		status = G_IO_STATUS_AGAIN;
 	}
-	else if (*bytes_read <= 0) {
+	else if (b_read > len) {
+		status = G_IO_STATUS_EOF;
+	}
+	else if (b_read < 0) {
 		status = G_IO_STATUS_ERROR;
 	} else {
+		*bytes_read = (guint) b_read;
 		status = G_IO_STATUS_NORMAL;
 	}
 
@@ -317,12 +321,12 @@ lm_ssl_new (const gchar    *expected_fingerprint,
  * 
  * Return value: A 16-byte array representing the fingerprint or %NULL if unknown.
  **/
-const unsigned char *
+const gchar *
 lm_ssl_get_fingerprint (LmSSL *ssl)
 {
 	g_return_val_if_fail (ssl != NULL, NULL);
 	
-	return (unsigned char*) ssl->fingerprint;
+	return ssl->fingerprint;
 }
 
 /**
